@@ -1,11 +1,11 @@
 //! `html_editor` is a simple html parser and editor.
 //!
-//! Example:
+//! Quick Start:
 //! ```
-//! use html_editor::prelude::*;
-//! use html_editor::{parse, Node, Selector};
+//! use html_editor::operation::*;
+//! use html_editor::{parse, Node};
 //!
-//! // You can create dom nodes by parsing html string.
+//! // You can create DOM nodes by parsing html string.
 //! let html = r#"
 //!     <!doctype html>
 //!     <html>
@@ -34,17 +34,21 @@
 //! ```
 
 mod data;
-mod edit;
-mod html;
 mod parse;
-mod query;
 
-pub mod prelude;
+pub mod operation;
 
 pub use parse::parse;
-pub use query::Selector;
+pub use parse::try_parse;
 
-/// Basic node of dom
+/// Doctype of Html or Xml
+#[derive(Clone, Debug)]
+pub enum Doctype {
+    Html,
+    Xml { version: String, encoding: String },
+}
+
+/// Node of DOM
 #[derive(Debug, Clone)]
 pub enum Node {
     Element {
@@ -54,15 +58,15 @@ pub enum Node {
     },
     Text(String),
     Comment(String),
-    Doctype,
+    Doctype(Doctype),
 }
 
 impl Node {
     /// Check if it is an element node.
-    /// 
+    ///
     /// ```
     /// use html_editor::Node;
-    /// 
+    ///
     /// assert_eq!(Node::new_element("div", vec![("id", "app")], vec![]).is_element(), true);
     /// assert_eq!(Node::Text("Lorem Ipsum".to_string()).is_element(), false);
     /// ```
@@ -73,19 +77,20 @@ impl Node {
         }
     }
 
-    /// Convert the node into an element. 
-    /// 
-    /// Note: The program will panic if it fails to convert.
+    /// Convert the node into an element.
+    ///
+    /// Warning: The program will panic if it fails to convert.
     /// So take care to use this method unless you are sure.
-    /// 
+    ///
+    /// Example:
     /// ```
     /// use html_editor::{Node, Element};
-    /// 
+    ///
     /// let a: Node = Node::new_element("div", vec![("id", "app")], vec![]);
     /// let a: Element = a.into_element();
-    /// 
+    ///
     /// let b: Node = Node::Text("hello".to_string());
-    /// // This will panic at 'Text("hello") is not an element'
+    /// // The next line will panic at 'Text("hello") is not an element'
     /// // let b: Element = a.into_element();
     /// ```
     pub fn into_element(self) -> Element {
@@ -104,12 +109,12 @@ impl Node {
     }
 
     /// Create a new element node.
-    /// 
+    ///
     /// ```
     /// use html_editor::Node;
-    /// 
+    ///
     /// let node: Node = Node::new_element(
-    ///     "h1", 
+    ///     "h1",
     ///     vec![("class", "title")],
     ///     vec![
     ///         Node::Text("Hello, world!".to_string()),
