@@ -1,3 +1,11 @@
+// TODO:
+//
+// Maybe we could have more elegant and concise code?
+// I will try to figure out a better way to parse when
+// am availible. Busy now for me :(
+//
+// If you have any idea, feel free to post an issue.
+
 mod attrs;
 mod token;
 
@@ -16,10 +24,9 @@ fn html_to_stack(html: &str) -> Result<Vec<Token>, String> {
     for ch in html.chars() {
         if let Some(quote) = in_quotes {
             if ch == quote {
-                let previous_char = chars_stack
+                let previous_char = *chars_stack
                     .last()
-                    .expect("cannot get the last char in chars stack")
-                    .clone();
+                    .expect("cannot get the last char in chars stack");
                 if previous_char != '\\' {
                     in_quotes = None;
                 }
@@ -38,7 +45,7 @@ fn html_to_stack(html: &str) -> Result<Vec<Token>, String> {
         } else if in_script {
             chars_stack.push(ch);
             let len = chars_stack.len();
-            
+
             if String::from_iter(&chars_stack).ends_with("</script>") {
                 let script = String::from_iter(chars_stack[..len - 9].to_vec());
                 chars_stack = Vec::new();
@@ -49,7 +56,7 @@ fn html_to_stack(html: &str) -> Result<Vec<Token>, String> {
         } else if in_style {
             chars_stack.push(ch);
             let len = chars_stack.len();
-            
+
             if String::from_iter(&chars_stack).ends_with("</style>") {
                 let style = String::from_iter(chars_stack[..len - 8].to_vec());
                 chars_stack = Vec::new();
@@ -62,7 +69,7 @@ fn html_to_stack(html: &str) -> Result<Vec<Token>, String> {
                 '<' => {
                     in_brackets = true;
                     // In case of pushing empty text tokens to the stack
-                    if chars_stack.len() != 0 {
+                    if !chars_stack.is_empty() {
                         // Turn the chars in `chars_stack` into `String`
                         // and clean the chars stack.
                         let txt_text = String::from_iter(chars_stack);
@@ -81,7 +88,7 @@ fn html_to_stack(html: &str) -> Result<Vec<Token>, String> {
                     chars_stack = Vec::new();
                     // Push the tag with the text we just got to the token stack.
                     let tag = Token::from(tag_text.clone())
-                        .expect(format!("Invalid tag: {}", tag_text).as_str());
+                        .unwrap_or_else(|| panic!("Invalid tag: {}", tag_text));
                     token_stack.push(tag.clone());
                     // Handle special tags
                     if let Token::Start(tag_name, _) = tag {
@@ -112,18 +119,22 @@ fn html_to_stack(html: &str) -> Result<Vec<Token>, String> {
             }
         }
     }
+    if !chars_stack.is_empty() {
+        let text = String::from_iter(chars_stack);
+        token_stack.push(Token::Text(text));
+    }
     Ok(token_stack)
 }
 
 fn stack_to_dom(token_stack: Vec<Token>) -> Result<Vec<Node>, String> {
     let mut nodes: Vec<Node> = Vec::new();
-    let mut start_tokens_stack: Vec<Token> = Vec::new();
-    let mut start_token_index = 0;
+    let mut start_tags_stack: Vec<Token> = Vec::new();
+    let mut start_tag_index = 0;
     for (i, token) in token_stack.iter().enumerate() {
         match token {
             Token::Start(tag, attrs) => {
                 let is_void_tag = VOID_TAGS.contains(&tag.as_str());
-                if start_tokens_stack.is_empty() {
+                if start_tags_stack.is_empty() {
                     if is_void_tag {
                         nodes.push(Node::Element {
                             name: tag.clone(),
@@ -131,22 +142,20 @@ fn stack_to_dom(token_stack: Vec<Token>) -> Result<Vec<Node>, String> {
                             children: Vec::new(),
                         });
                     } else {
-                        start_token_index = i;
-                        start_tokens_stack.push(Token::Start(tag.clone(), attrs.clone()));
+                        start_tag_index = i;
+                        start_tags_stack.push(Token::Start(tag.clone(), attrs.clone()));
                     }
+                } else if is_void_tag {
+                    // You do not need to push the void tag to the stack
+                    // like above, because it must be inside the the
+                    // element of the first start tag, and this element
+                    // will then be pushed to the stack recursively.
                 } else {
-                    if is_void_tag {
-                        // You do not need to push the void tag to the stack
-                        // like above, because it must be inside the the
-                        // element of the first start tag, and this element
-                        // will then be pushed to the stack recursively.
-                    } else {
-                        start_tokens_stack.push(Token::Start(tag.clone(), attrs.clone()));
-                    }
+                    start_tags_stack.push(Token::Start(tag.clone(), attrs.clone()));
                 }
             }
             Token::End(tag) => {
-                let start_tag = match start_tokens_stack.pop() {
+                let start_tag = match start_tags_stack.pop() {
                     Some(token) => token.into_node().into_element(),
                     None => return Err(format!("No start tag matches </{}>", tag)),
                 };
@@ -156,23 +165,23 @@ fn stack_to_dom(token_stack: Vec<Token>) -> Result<Vec<Node>, String> {
                         start_tag.name, tag
                     ));
                 }
-                if start_tokens_stack.is_empty() {
+                if start_tags_stack.is_empty() {
                     nodes.push(Node::Element {
                         name: start_tag.name,
                         attrs: start_tag.attrs,
-                        children: stack_to_dom(token_stack[start_token_index + 1..i].to_vec())?,
+                        children: stack_to_dom(token_stack[start_tag_index + 1..i].to_vec())?,
                     })
                 }
             }
             _ => {
-                if start_tokens_stack.is_empty() {
+                if start_tags_stack.is_empty() {
                     nodes.push(token.node());
                 }
             }
         }
     }
 
-    match start_tokens_stack.pop() {
+    match start_tags_stack.pop() {
         Some(token) => {
             let start_tag_name = token.into_node().into_element().name;
             Err(format!("<{}> is not closed", start_tag_name))
@@ -181,6 +190,81 @@ fn stack_to_dom(token_stack: Vec<Token>) -> Result<Vec<Node>, String> {
     }
 }
 
+fn try_stack_to_dom(token_stack: Vec<Token>) -> Vec<Node> {
+    let mut nodes: Vec<Node> = Vec::new();
+    let mut start_tags_stack: Vec<Token> = Vec::new();
+    let mut start_tag_index = 0;
+
+    for (i, token) in token_stack.iter().enumerate() {
+        match token {
+            Token::Start(tag, attrs) => {
+                let is_void_tag = VOID_TAGS.contains(&tag.as_str());
+                if start_tags_stack.is_empty() {
+                    if is_void_tag {
+                        nodes.push(Node::Element {
+                            name: tag.clone(),
+                            attrs: attrs.clone(),
+                            children: Vec::new(),
+                        });
+                    } else {
+                        start_tag_index = i;
+                        start_tags_stack.push(Token::Start(tag.clone(), attrs.clone()));
+                    }
+                } else if is_void_tag {
+                    // You do not need to push the void tag to the stack
+                    // like above, because it must be inside the the
+                    // element of the first start tag, and this element
+                    // will then be pushed to the stack recursively.
+                } else {
+                    start_tags_stack.push(Token::Start(tag.clone(), attrs.clone()));
+                }
+            }
+            Token::End(tag) => {
+                let start_tag = match start_tags_stack.pop() {
+                    Some(token) => token.into_node().into_element(),
+                    // It means the end tag is redundant, so we will omit
+                    // it and just start the next loop.
+                    None => continue,
+                };
+
+                if tag != &start_tag.name {
+                    // The tags do not match, so let's put it back to
+                    // pretend we never come here and then continue
+                    // the next loop.
+                    start_tags_stack.push(Token::Start(start_tag.name, start_tag.attrs));
+                    continue;
+                }
+
+                if start_tags_stack.is_empty() {
+                    nodes.push(Node::Element {
+                        name: start_tag.name,
+                        attrs: start_tag.attrs,
+                        children: try_stack_to_dom(token_stack[start_tag_index + 1..i].to_vec()),
+                    })
+                }
+            }
+            _ => {
+                if start_tags_stack.is_empty() {
+                    nodes.push(token.node());
+                }
+            }
+        }
+    }
+
+    while let Some(token) = start_tags_stack.pop() {
+        let node = match token {
+            Token::Start(name, attrs) => Node::Element {
+                name,
+                attrs,
+                children: try_stack_to_dom(token_stack[start_tag_index + 1..].to_vec()),
+            },
+            _ => unreachable!(),
+        };
+        nodes = vec![node];
+    }
+    nodes
+}
+
 /// Parse the html string and return a `Vector` of `Node`.
 ///
 /// Example:
@@ -195,7 +279,7 @@ fn stack_to_dom(token_stack: Vec<Token>) -> Result<Vec<Node>, String> {
 /// let document = parse("<!doctype html><html><head></head><body></body></html>");
 /// println!("{:#?}", document);
 /// ```
-/// 
+///
 /// Output:
 /// ```log
 /// [
@@ -212,7 +296,7 @@ fn stack_to_dom(token_stack: Vec<Token>) -> Result<Vec<Node>, String> {
 ///     },
 /// ]
 /// [
-///     Doctype,
+///     Doctype(Html),
 ///     Element {
 ///         name: "html",
 ///         attrs: {},
@@ -234,6 +318,29 @@ fn stack_to_dom(token_stack: Vec<Token>) -> Result<Vec<Node>, String> {
 pub fn parse(html: &str) -> Result<Vec<Node>, String> {
     let stack = html_to_stack(html)?;
     // println!("{:#?}", stack);
-    let dom = stack_to_dom(stack);
-    dom
+    
+    stack_to_dom(stack)
+}
+
+/// Alternative for [`parse()`](parse) with fault tolerance
+/// feature.
+/// 
+/// Whatever the input is, it will try to return a vector of nodes
+/// without errors. It can parse some illegal html code like `<div><a>Ipsum`
+/// or `<div>Ipsum</a>`.
+///
+/// But we still suggest you to use [`parse()`](parse) unless neccessary for better
+/// error handling.
+/// 
+/// ```
+/// use html_editor::{try_parse, operation::Htmlifiable};
+/// 
+/// let result: String = try_parse("<div><a>Ipsum</div>").html();
+/// assert_eq!(result, "<div><a>Ipsum</a></div>");
+/// ```
+pub fn try_parse(html: &str) -> Vec<Node> {
+    let stack = html_to_stack(html).unwrap_or_default();
+    // println!("{:?}", stack);
+    
+    try_stack_to_dom(stack)
 }
